//--------------------------------------------------------------------
//
//  Laboratory 3                                         listarr.cpp
//
//  SOLUTION: Array implementation of the List ADT
//
//--------------------------------------------------------------------


#include "listarr.h"

using namespace std;

//--------------------------------------------------------------------

List:: List ( int maxNumber )

// Creates an empty list. Allocates enough memory for maxNumber
// data items (defaults to defMaxListSize).

  : maxSize(maxNumber),
    size(0),
    cursor(-1)
{
   dataItems = new DataType[maxSize];
}

//--------------------------------------------------------------------

List:: ~List ()

// Frees the memory used by a list.

{
    delete [] dataItems;
}

//--------------------------------------------------------------------

void List:: insert ( const DataType &newDataItem ) 
    throw ( logic_error )

// Inserts newDataItem after the cursor. If the list is empty, then
// newDataItem is inserted as the first (and only) data items in the
// list. In either case, moves the cursor to newDataItem.

{
    int j;   // Loop counter

    // Requires that list is not full
    if ( size >= maxSize )
        throw logic_error("list is full\n");

    for ( j = size ; j > cursor+1 ; j-- )
        dataItems[j] = dataItems[j-1];
    size++;
    dataItems[++cursor] = newDataItem;
}

//--------------------------------------------------------------------

void List:: remove () throw ( logic_error )

// Removes the data items  marked by the cursor from a list. Moves the
// cursor to the next data item item in the list. Assumes that the
// first list data items "follows" the last list data item.

{
    int j;   // Loop counter

    // Requires that the list is not empty
    if ( size == 0 )
    	throw logic_error("size == 0\n");

    for ( j = cursor ; j < size-1 ; j++ )
        dataItems[j] = dataItems[j+1];
    size--;
    if ( size == 0 )
       cursor = -1;
    else if ( size == cursor )
       cursor = 0;
}

//--------------------------------------------------------------------

void List:: replace ( const DataType &newDataItem )
    throw ( logic_error )

// Replaces the item marked by the cursor with newDataItem and
// leaves the cursor at newDataItem.

{
    // Requires that the list is not empty
    if ( size == 0 )
    	throw logic_error("size == 0\n");

    dataItems[cursor] = newDataItem;
}

//--------------------------------------------------------------------

void List:: clear ()

// Removes all the data items from a list.

{
    size = 0;
    cursor = -1;
}

//--------------------------------------------------------------------

bool List:: isEmpty () const

// Returns 1 if a list is empty. Otherwise, returns 0.

{
    return ( size == 0 );
}

//--------------------------------------------------------------------

bool List:: isFull () const

// Returns 1 if a list is full. Otherwise, returns 0.

{
    return ( size == maxSize );
}

//--------------------------------------------------------------------

void List:: gotoBeginning () throw ( logic_error )

// Moves the cursor to the beginning of the list.

{
   if ( size == 0 )
      throw logic_error("list is empty");
   
   cursor = 0;
}

//--------------------------------------------------------------------

void List:: gotoEnd () throw ( logic_error )

// Moves the cursor to the end of the list.

{
   if ( size == 0 )
      throw logic_error("list is empty");
   
   cursor = size - 1;
}

//--------------------------------------------------------------------

bool List:: gotoNext () throw ( logic_error )

// If the cursor is not at the end of a list, then moves the
// cursor to the next item in the list and returns true. Otherwise,
// returns false.

{
   if ( size == 0 )
      throw logic_error("list is empty");

   if ( cursor != size-1 )
   {
      cursor++;
      return true;
   }
   else
      return false;
}

//--------------------------------------------------------------------

bool List:: gotoPrior () throw ( logic_error )

// If the cursor is not at the beginning of a list, then moves the
// cursor to the preceeding item in the list and returns true.
// Otherwise, returns false.

{
   if ( size == 0 )
      throw logic_error("list is empty");

   if ( cursor != 0 )
   {
      cursor--;
      return true;
   }
   else
      return false;
}

//--------------------------------------------------------------------

DataType List:: getCursor () const throw ( logic_error )

// Returns the item marked by the cursor.

{
    if ( size == 0 )
    	throw logic_error("list is empty\n");  // Requires that the list is not empty

    return dataItems[cursor];
}

//--------------------------------------------------------------------

void List:: showStructure () const

// Outputs the data items in a list. If the list is empty, outputs
// "Empty list". This operation is intended for testing/debugging
// purposes only.

{
    int j;   // Loop counter

    if ( size == 0 )
       cout << "Empty list" << endl;
    else
    {
       cout << "size = " << size
            <<  "   cursor = " << cursor << endl;
       for ( j = 0 ; j < maxSize ; j++ )
           cout << j << "\t";
       cout << endl;
       for ( j = 0 ; j < size ; j++ )
           cout << dataItems[j] << "\t";
       cout << endl;
    }
}

//--------------------------------------------------------------------
//
//                        In-lab operations
//
//--------------------------------------------------------------------
bool List:: find ( const DataType &searchDataItem ) throw ( logic_error )

// Searches a list for searchDataItem. Begins the search with the
// data items marked by the cursor. Moves the cursor through the list
// until either searchDataItem is found (returns true) or the end of the
// list is reached (returns false).
// The find function should print a message that the searchDataItem has been
// found and the cursor position where it has been found. For instance:
// "found 'a' at position 3" or "didn't find 'b' the cursor is 3" 

{
	
	do 
	{
      //****Add code here to implement the details above.
      //Begins the search with the
      // data items marked by the cursor. Moves the cursor through the list
      // until either searchDataItem is found (returns true)
	}while (gotoNext());
	cout << "didn't find "<< searchDataItem << " the cursor is " << cursor << endl;
	return false;  //for now
}

